<?php

namespace Drupal\gtext\Form;

use Drupal\Component\Gettext\PoItem;
use \Drupal\Core\Form\FormBase;
use \Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\locale\SourceString;

class TranslateStringForm extends FormBase {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'form_translate_string_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [
      '#id'       => 'form__translate_string_form',
      '#attached' => [
        'library'   => ['gtext/gtext.string']
      ],
    ];


    $lid    = $form_state->getBuildInfo()['args'][0]->getId();
    if ( $form_state->getValue(['lid']) ){
      $lid    = $form_state->getValue(['lid']);
    }
    if ( !empty($_SESSION['gtext'][$lid]) ){
      $form['listening'] = [
        '#type'  => 'html_tag',
        '#tag'   => 'script',
        '#value' => 'if ( opener ) opener.reloadTranslations('.$lid.');',
      ];
      unset($_SESSION['gtext'][$lid]);
    }
    $form['lid'] = [
      '#type'     => 'hidden',
      '#value'    => $lid,
    ];

    $originalSource = new SourceString($form_state->getBuildInfo()['args'][0]);
    $originalString = $originalSource->getPlurals();

    $form['string'] = [
      '#tree'   => true,
    ];
    foreach(\Drupal::languageManager()->getLanguages() as $langcode => $language){
      if ( !\locale_is_translatable($langcode)) {
          continue;
      }

      $string = \Drupal::service('locale.storage')->getTranslations([
        'lid'        => $lid,
        'language'   => $langcode,
        'translated' => TRUE
      ]);

      $source_array = [];
      if ( $string ){
        $string = reset($string);
        $source_array = $string->getPlurals();
      }

      if ( count($originalString) == 1){
        $form['string'][$langcode] = [
          '#type'          => 'textarea',
          '#title'         => $language->getName(),
          '#default_value' => $source_array[0] ?? '',
          '#weight'        => $language->getWeight(),
          '#attributes'    => [
            'class'          => ['gtext-translatable-field'],
            'id'             => 'translation--'.$langcode,
            'data-lang'      => $langcode,
            'data-text'      => $originalString[0],
            'data-url'       => Url::fromRoute('gtext.translate.google')->toString()
          ],
        ];
      } else {
        $pluralCount = (int) $this->getNumberOfPlurals($langcode);
        $form['string'][$langcode] = [
          '#type'          => 'details',
          '#open'          => true,
          '#title'         => $language->getName(),
          '#weight'        => $language->getWeight(),
          '#attributes'    => [
            'data-lang'         => $langcode,
            'data-plural-count' => $pluralCount
          ]
        ];
        for ($i = 0; $i < $pluralCount; $i++) {
          $form['string'][$langcode][$i] = [
            '#title'            => ($i == 0 ? $this->t('Singular form') : $this->formatPlural($i, 'First plural form', '@count. plural form')),
            '#type'             => 'textarea',
            '#rows'             => 2,
            '#default_value'    => $source_array[$i] ?? '',
            '#parents'          => ['string', $langcode, $i],
            '#weight'        => $i * 2 + 1,
            '#attributes'    => [
              'id'             => 'translation--' . $langcode.'--' . $i,
              'data-lang'      => $langcode,
              'data-text'      => $originalString[$i == 0 ? 0 : 1],
              'data-url'       => Url::fromRoute('gtext.translate.google')->toString(),
              'class'          => ['gtext-translatable-field'],
            ],
          ];
        }
        if ($pluralCount == 2) {
          $form['string'][$langcode][1]['#title'] = $this->t('Plural form');
        }
      }
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type'        => 'submit',
      '#id'          => 'submit_form',
      '#value'       => gtext()->gtext('Save'),
      '#button_type' => 'primary',
    ];
    $form['actions']['submit_close'] = [
      '#type'        => 'submit',
      '#id'          => 'submit_form_close',
      '#value'       => gtext()->gtext('Save and close'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
  * {@inheritdoc}
  */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $lid = $form_state->getValue(['lid']);

    $strings = $form_state->getValue(['string']);
    foreach(\Drupal::languageManager()->getLanguages() as $langcode => $language){
      if ( !isset($strings[$langcode]) ){
        continue;
      }
      $string = $strings[$langcode];
      if ( $langcode == 'en' && !\locale_is_translatable('en') ) {
        continue;
      }
      $string_delimited = is_string($string) ?  $string : implode(PoItem::DELIMITER,  $string);

      $translation = \Drupal::service('locale.storage')->getTranslations([
        'lid'        => $lid,
        'language'   => $langcode,
        'translated' => TRUE
      ]);
      $translation = !empty($translation) ? reset($translation) : false;

      if ( !$translation || $translation->getValues(['source'])['source'] != $string_delimited ){
        // Changed -> save
        $target = $translation !== false ? $translation : \Drupal::service('locale.storage')->createTranslation(['lid' => $lid, 'language' => $langcode]);
        $target->setPlurals(is_string($string) ? [$string] : $string)
          ->setCustomized()
          ->save();

        $_SESSION['gtext'][$lid] = true;
        _locale_refresh_translations([$langcode], [$lid]);
        _locale_refresh_configuration([$langcode], [$lid]);
      }
      if ( (is_string($string) && empty($string)) || (is_array($string) && count(array_filter($string)) == 0) ){
        // Empty translations -> delete translation
        if ( $translation ){
          $_SESSION['gtext'][$lid] = true;
          $translation->delete();
          _locale_refresh_translations([$langcode], [$lid]);
          _locale_refresh_configuration([$langcode], [$lid]);
        }
      }
    }
    $button = $form_state->getTriggeringElement();
    if ( $button['#id'] == 'submit_form_close' ){
      $form_state->setRedirect('gtext.translate.string', ['lid' => $lid], ['query' => ['action' => 'close']]);
    } else {
      $form_state->setRedirect('gtext.translate.string', ['lid' => $lid]);
    }
    $this->messenger()->addStatus(gtext()->gtext('Saved'));
  }
  /**
   * Ajax callback
   */
  public static function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form;
  }
}